//====================================================================
//    IrDA.    
//IrDA,       IrDA,   , 
// ,     .
//====================================================================
class IrDAFileSend
{
private string m_descriptionOfLastSendAttempt;     
private string m_IrDAServiceName;
private string m_fileToSend;
private bool m_wasSenderStopped;           

public enum SendStatus
{
  AttemptingToSend, 
  Finished_Successfully,
  Finished_Aborted,
  Finished_Error
}

private SendStatus m_SendStatus;           
public SendStatus Status
{           
  get 
  { 
    //    /  m_SendStatus 
    lock(this) {return m_SendStatus;}
  }
}
private void setStatus(SendStatus newStatus)
{
    //    /  m_SendStatus 
    lock(this) {m_SendStatus = newStatus;}
}

public string ErrorText
{
  get {return m_descriptionOfLastSendAttempt;}
}

//------------------------------------------------------------
//
//------------------------------------------------------------
public IrDAFileSend(string fileToSend, string irdaServiceName)
{
  //  IrDA,     
  m_IrDAServiceName = irdaServiceName;
  //,    
  m_fileToSend = fileToSend;
}

//--------------------------------------------------------------
//       
//--------------------------------------------------------------
public void LoopAndAttemptIRSendAsync()
{
  //    
  setStatus(SendStatus.AttemptingToSend); 
  //      
  m_wasSenderStopped = false; 

  // ,       
  System.Threading.ThreadStart threadEntryPoint;
  threadEntryPoint = 
    new System.Threading.ThreadStart(LoopAndAttemptIRSend);

  //--------------------------------------------
  //     
  //--------------------------------------------
  System.Threading.Thread newThread =
    new System.Threading.Thread(threadEntryPoint);
    
  newThread.Start(); //!
}

//-------------------------------------------------------------
//        IR
//-------------------------------------------------------------
public void LoopAndAttemptIRSend()
{
  System.Net.Sockets.IrDAClient irDASender;           
  System.IO.Stream streamOutToIrDA ;
  System.IO.Stream streamInFromFile;

  //      
  m_wasSenderStopped = false; 
  setStatus(SendStatus.AttemptingToSend);

  //-----------------------------------------------------------------
  //  ,      
  //-----------------------------------------------------------------
  while(true)
  {
    //          
    // 'sendStream(...)',     !
    irDASender = null;           
    streamOutToIrDA = null;
    streamInFromFile = null;

    //  
    bool bSuccess; 
    try
    {
      bSuccess = sendStream( 
        out m_descriptionOfLastSendAttempt,
        ref streamOutToIrDA,
        ref irDASender,
        ref streamInFromFile
        );
    }
    catch(System.Exception eUnexpected) // !!!
    {
      setStatus(SendStatus.Finished_Error); //  
      m_descriptionOfLastSendAttempt = 
        "Unexpected error in IR send loop. " + 
        eUnexpected.Message;

      //---------------------------------------------------
      //     
      //---------------------------------------------------
      if(streamOutToIrDA != null)
      {
        try {streamOutToIrDA.Close();} 
        catch{};//  
        streamOutToIrDA = null;
      }

      if(streamInFromFile != null)
      {
        try {streamInFromFile.Close();} 
        catch{};//  
        streamInFromFile = null;
      }

      if (irDASender != null)
      {
        try {irDASender.Close();} 
        catch{}; //  
        irDASender = null;
      }
      return; //
    }

    //  
    if (bSuccess == true)
    {
      m_descriptionOfLastSendAttempt = "Success!";
      setStatus(SendStatus.Finished_Successfully);
      return;
    }

    //,      
    if (m_wasSenderStopped == true)
    {
      m_descriptionOfLastSendAttempt = "User Aborted.";
      setStatus(SendStatus.Finished_Aborted);
      return;
    }

    //  ...       IrDA,
    //     .   
    //    .
  }

  //         
}

//-----------------------------------------------------------------------
//   - (, )  IR 
//[ ]: 
//   true:   
//   false:     
//-----------------------------------------------------------------------
private bool sendStream(out string errorDescription,
  ref System.IO.Stream streamOutToIrDA,
  ref System.Net.Sockets.IrDAClient irDASender,
  ref System.IO.Stream streamInFromFile
)
{
  errorDescription = "";
  //----------------------------------------------------------
  //   IRDA
  //----------------------------------------------------------
  try
  {
    //--------------------------------------------------------
    //   .    
    // ,    .
    //--------------------------------------------------------
    irDASender = 
      new System.Net.Sockets.IrDAClient(m_IrDAServiceName);
  }

  catch (System.Exception eCreateClient)
  {
    //      :
    //#1:   
    //#2:   ,    
    //   (   )
    errorDescription = eCreateClient.Message;
    return false;
  }

  //      :
  //#1:       IR
  //#2: IR-   (-   STOP). 
  if (m_wasSenderStopped == true)
  {
    irDASender.Close();
    irDASender = null;
    return false;
  }

  //==========================================
  // !
  //==========================================
  // ,    
  streamInFromFile = System.IO.File.OpenRead(m_fileToSend);
  //  IrDA,     
  streamOutToIrDA = irDASender.GetStream();

  const int BUFFER_SIZE = 1024;
  byte[] inBuffer = new byte[BUFFER_SIZE]; 
  int bytesRead;
  int iTestAll = 0;
  int iTestWrite = 0;
  do
  {
    //    
    bytesRead = streamInFromFile.Read(inBuffer, 0, BUFFER_SIZE);
    iTestAll = iTestAll + 1;

    //     
    if (bytesRead > 0)
    {
      streamOutToIrDA.Write(inBuffer, 0, bytesRead);
      iTestWrite = iTestWrite + 1;
    }

  }while(bytesRead > 0);

  //  
  streamOutToIrDA.Flush(); //   
  streamOutToIrDA.Close(); // 
  streamOutToIrDA = null;

  //  
  streamInFromFile.Close();
  streamOutToIrDA  = null;

  //  IrDA
  irDASender.Close();
  irDASender = null;

  // !!!
  return true;
}
} // 
