Option Strict On
'====================================================================
'    IrDA.    
'IrDA,       IrDA,   , 
' ,     .
'====================================================================
Class IrDAFileSend
Private m_descriptionOfLastSendAttempt As String
Private m_IrDAServiceName As String
Private m_fileToSend As String
Private m_wasSenderStopped As Boolean

Public Enum SendStatus
  AttemptingToSend
  Finished_Successfully
  Finished_Aborted
  Finished_Error
End Enum

Private m_SendStatus As SendStatus
Public ReadOnly Property Status() As SendStatus
  Get
    '    /  m_SendStatus
    SyncLock (Me)
      Return m_SendStatus
    End SyncLock
  End Get
End Property
Private Sub setStatus(ByVal newStatus As SendStatus)
    '    /  m_SendStatus
    SyncLock (Me)
      m_SendStatus = newStatus
    End SyncLock
End Sub

Public ReadOnly Property ErrorText() As String
Get
  Return m_descriptionOfLastSendAttempt
End Get
End Property

'------------------------------------------------------------
'
'------------------------------------------------------------
Public Sub New(ByVal fileToSend As String, ByVal irdaServiceName As String)
  '  IrDA,     
  m_IrDAServiceName = irdaServiceName
  ',    
  m_fileToSend = fileToSend
End Sub

'--------------------------------------------------------------
'       
'--------------------------------------------------------------
Public Sub LoopAndAttemptIRSendAsync()
  '    
  setStatus(SendStatus.AttemptingToSend)
  '      
  m_wasSenderStopped = False

  ' ,       
  Dim threadEntryPoint As System.Threading.ThreadStart
  threadEntryPoint = _
    New System.Threading.ThreadStart(AddressOf LoopAndAttemptIRSend)

  '--------------------------------------------
  '     
  '--------------------------------------------
  Dim newThread As System.Threading.Thread = _
    New System.Threading.Thread(threadEntryPoint)

  newThread.Start() '!
End Sub

'-------------------------------------------------------------
'        IR
'-------------------------------------------------------------
Public Sub LoopAndAttemptIRSend()
  Dim irDASender As System.Net.Sockets.IrDAClient
  Dim streamOutToIrDA As System.IO.Stream
  Dim streamInFromFile As System.IO.Stream

  '      
  m_wasSenderStopped = False
  setStatus(SendStatus.AttemptingToSend)

  '-----------------------------------------------------------------
  '  ,     
  '-----------------------------------------------------------------
  While (True)
    '          
    ' sendStream(...),     !
    irDASender = Nothing
    streamOutToIrDA = Nothing
    streamInFromFile = Nothing

    '  
    Dim bSuccess As Boolean
    Try
      bSuccess = sendStream(m_descriptionOfLastSendAttempt, _
        streamOutToIrDA, irDASender, streamInFromFile)

    Catch eUnexpected As System.Exception ' !!!
      setStatus(SendStatus.Finished_Error) '  

      m_descriptionOfLastSendAttempt = _
    "Unexpected error in IR send loop. " + eUnexpected.Message

      '---------------------------------------------------
      '     
      '---------------------------------------------------
      If Not (streamOutToIrDA Is Nothing) Then
        Try
            streamOutToIrDA.Close()
        Catch
         '  
        End Try
        streamOutToIrDA = Nothing
      End If

      If Not (streamInFromFile Is Nothing) Then
        Try
          streamInFromFile.Close()
        Catch
          '  
        End Try
        streamInFromFile = Nothing
      End If

      If Not (irDASender Is Nothing) Then
        Try
          irDASender.Close()
        Catch
          '  
        End Try
        irDASender = Nothing
      End If
      Return '
    End Try

    '  
    If (bSuccess = True) Then
      m_descriptionOfLastSendAttempt = "Success!"
      setStatus(SendStatus.Finished_Successfully)
      Return
    End If

    ',      
    If (m_wasSenderStopped = True) Then
      m_descriptionOfLastSendAttempt = "User Aborted."
      setStatus(SendStatus.Finished_Aborted)
      Return
    End If

    '  ...       IrDA,
    '     .     
    '   .
  End While

  '         
End Sub

'----------------------------------------------------------------------
'   - (, )  IR 
'[ ]: 
'  true:   
'  false:     
'----------------------------------------------------------------------
Private Function sendStream(ByRef errorDescription As String, _
  ByRef streamOutToIrDA As System.IO.Stream, _
  ByRef irDASender As System.Net.Sockets.IrDAClient, _
  ByRef streamInFromFile As System.IO.Stream) As Boolean

  errorDescription = ""
  '----------------------------------------------------------
  '   IRDA
  '----------------------------------------------------------
  Try
    '--------------------------------------------------------
    '   .    
    ' ,    .
    '--------------------------------------------------------
    irDASender = _
      New System.Net.Sockets.IrDAClient(m_IrDAServiceName)

  Catch eCreateClient As System.Exception
    '      :
    '#1:   
    '#2:   ,    
    '  (   )
    errorDescription = eCreateClient.Message
    Return False
  End Try

  '      :
  '#1:       IR
  '#2: IR-   (-   STOP). 
  If (m_wasSenderStopped = True) Then
    irDASender.Close()
    irDASender = Nothing
    Return False
  End If

  '==========================================
  ' !
  '==========================================
  ' ,    
  streamInFromFile = System.IO.File.OpenRead(m_fileToSend)
  '  IrDA,     
  streamOutToIrDA = irDASender.GetStream()

  Const BUFFER_SIZE As Integer = 1024
  Dim inBuffer() As Byte
  ReDim inBuffer(BUFFER_SIZE)
  Dim bytesRead As Integer
  Dim iTestAll As Integer
  Dim iTestWrite As Integer
  '...
  Do
    '    
    bytesRead = streamInFromFile.Read(inBuffer, 0, BUFFER_SIZE)
    iTestAll = iTestAll + 1

    '     
    If (bytesRead > 0) Then
          streamOutToIrDA.Write(inBuffer, 0, bytesRead)
      iTestWrite = iTestWrite + 1
    End If

  Loop While (bytesRead > 0)

  '  
  streamOutToIrDA.Flush() '   
  streamOutToIrDA.Close() ' 
  streamOutToIrDA = Nothing

  '  
  streamInFromFile.Close()
  streamOutToIrDA = Nothing

  '  IrDA
  irDASender.Close()
  irDASender = Nothing

  ' !!!
  Return True
End Function
End Class
