//-------------------------------------------------------------------
//    IrDA ( ).
//
//          
//     .     
//   IR,     
//    .
//-------------------------------------------------------------------
public class IrDAFileReceive
{
private bool m_wasListenerStopped;
private string m_IrDAServiceName;
private string m_fileNameForDownload;
private string m_errorDuringTransfer;
private System.Net.Sockets.IrDAListener m_irListener;
private ReceiveStatus m_ReceiveStatus;

public string ErrorText
{get {return m_errorDuringTransfer;}} 
//--------------------------------------------------------------
//  
//--------------------------------------------------------------
public enum ReceiveStatus
{
  NotDone_SettingUp,
  NotDone_WaitingForSender,
  NotDone_Receiving,
  Done_Success,
  Done_Aborted,
  Done_ErrorOccured
} 

//---------------------------------------------------------
//   
//---------------------------------------------------------
public ReceiveStatus Status
{
  get 
  {
    //    
    //   /
    lock(this)  
    {
      return m_ReceiveStatus;
    } // lock
  } // get
} // 

private void setStatus(ReceiveStatus newStatus)
{
  //    
  //   /
  lock(this)  
  {
    m_ReceiveStatus = newStatus;
  } // lock
} 

//-------------------------------------------------------------
// [in] filename:      IR
//-------------------------------------------------------------
public IrDAFileReceive(string filename, string irdaServiceName)
{
  //  IrDA,    
  m_IrDAServiceName = irdaServiceName;

  // ,       
  m_fileNameForDownload = filename;
} 

//-----------------------------------------------------------
//     IR
//
// [in] filename:  ,    
//-----------------------------------------------------------
public void WaitForIRFileDownloadAsync()
{
  //,         
  setStatus(ReceiveStatus.NotDone_SettingUp);
  //-----------------------------------------------------------
  //  
  //-----------------------------------------------------------
  System.Threading.ThreadStart threadEntryPoint;
  threadEntryPoint = 
    new System.Threading.ThreadStart(WaitForIRFileDownload);

  System.Threading.Thread newThread = 
    new System.Threading.Thread(threadEntryPoint);

  //   
  newThread.Start();
} 

//-----------------------------------------------------------
//  IR    
//-----------------------------------------------------------
public void WaitForIRFileDownload()
{
  System.IO.Stream outputStream = null;
  System.Net.Sockets.IrDAClient irdaClient = null;
  System.IO.Stream irStreamIn = null;

  try
  {
    //=========================================================
    //   !
    //=========================================================
    internal_WaitForIRFileDownload(ref outputStream, ref irdaClient,
                ref irStreamIn);
  }  
  catch //    
  {
    setStatus(ReceiveStatus.Done_ErrorOccured);
  } 

  //=============================================
  //  
  //=============================================
  //   
  if(irStreamIn != null)
  {
    try {irStreamIn.Close();}
    catch {} //   
  }
  //  IrDA
  if(irdaClient != null)
  {
    try {irdaClient.Close();}
    catch {} //   
  }
  // ,    
  if(outputStream != null)
  {
    try {outputStream.Close();}
    catch {} //   
  }
  //  ,   
  if (m_irListener != null)
  {
    // ,  ,   ,
    // ,   
    m_wasListenerStopped = true;
    try {m_irListener.Stop();} catch{} //   
    m_irListener = null;
  } 
}

private void internal_WaitForIRFileDownload(
    ref System.IO.Stream outputStream,
    ref System.Net.Sockets.IrDAClient irdaClient,
    ref System.IO.Stream irStreamIn
)
{
  //----------------------------------------------------------
  //        
  //----------------------------------------------------------
  outputStream = System.IO.File.Open(
    m_fileNameForDownload, 
    System.IO.FileMode.Create);

  //==========================================
  // 
  //==========================================
  setStatus(ReceiveStatus.NotDone_WaitingForSender);

  //-------------------------------------------------------------
  //  
  //-------------------------------------------------------------
  try
  {
    m_wasListenerStopped = false;
    m_irListener = 
      new System.Net.Sockets.IrDAListener(m_IrDAServiceName);
    m_irListener.Start();
  }
  catch(System.Exception eListener)
  {
    m_errorDuringTransfer = "Error creating listener - " + 
      eListener.Message;
    goto exit_sub_with_error;
  } 

  //,      
  if (m_wasListenerStopped == true)
  { goto exit_sub_with_abort; }

  //---------------------------------------------------------
  // 
  //---------------------------------------------------------
  try
  {
    //--------------------------------------------------------------------
    //      ,    
    //  ,     
    //,    )            
    //--------------------------------------------------------------------
    irdaClient = m_irListener.AcceptIrDAClient();
  }
  catch(System.Exception eClientAccept)
  {
    //    ,   
    //,      
    // . 
    if (m_wasListenerStopped == true)
    {goto exit_sub_with_abort; }

    //    , 
    //   .  .
    m_errorDuringTransfer = "Error accepting connection - " + 
      eClientAccept.Message;

    goto exit_sub_with_error;
  } 
  //     :
  //#1:       IR
  //#2: IR-   (-   STOP)
  //    (       )
  //,     
  if (m_wasListenerStopped == true)
  {goto exit_sub_with_abort;} 

  //==========================================
  // 
  //==========================================
  setStatus(ReceiveStatus.NotDone_Receiving);

  //--------------------------------------------------------------
  //  
  //--------------------------------------------------------------
  try
  {
    irStreamIn = irdaClient.GetStream();
  }
  catch(System.Exception exGetInputStream)
  {
    m_errorDuringTransfer = "Error getting input stream - " +
      exGetInputStream.Message;
    goto exit_sub_with_error;
  } 
  //   !
  const int BUFFER_SIZE = 1024;
  byte[] inBuffer = new byte[BUFFER_SIZE];
  int bytesRead = 0;
  do
  {
    //    IR
    bytesRead = irStreamIn.Read(inBuffer, 0, BUFFER_SIZE);

    //     
    if (bytesRead > 0)
    {outputStream.Write(inBuffer, 0, bytesRead);}
  }while (bytesRead > 0);

  outputStream.Flush(); //    

  //==========================================
  // :  
  //==========================================
  setStatus(ReceiveStatus.Done_Success);
  return; // 

  //==========================================
  //...
  //==========================================
exit_sub_with_abort:
  // :  (  - )
  setStatus(ReceiveStatus.Done_Aborted);
  return;

exit_sub_with_error:

  // : !!!!
  setStatus(ReceiveStatus.Done_ErrorOccured);
}
} // 
