'-------------------------------------------------------------------
'    IrDA ( )
'
'         
'     .     
'   IR,     
'    .
'-------------------------------------------------------------------
Public Class IrDAFileReceive

Private m_wasListenerStopped As Boolean
Private m_IrDAServiceName As String
Private m_fileNameForDownload As String
Private m_errorDuringTransfer As String
Private m_irListener As System.Net.Sockets.IrDAListener
Private m_ReceiveStatus As ReceiveStatus

Public ReadOnly Property ErrorText() As String
Get
  Return m_errorDuringTransfer
End Get
End Property

'--------------------------------------------------------------
'  
'--------------------------------------------------------------
Public Enum ReceiveStatus
  NotDone_SettingUp
  NotDone_WaitingForSender
  NotDone_Receiving
  Done_Success
  Done_Aborted
  Done_ErrorOccured
End Enum

'---------------------------------------------------------
'   
'---------------------------------------------------------
Public ReadOnly Property Status() As ReceiveStatus
Get
  SyncLock (Me)
        Return m_ReceiveStatus
  End SyncLock
End Get
End Property

Private Sub setStatus(ByVal newStatus As ReceiveStatus)
  '    
  '   /
  SyncLock (Me)
      m_ReceiveStatus = newStatus
  End SyncLock 'end lock
End Sub

'-------------------------------------------------------------
' (in) filename:      IR
'-------------------------------------------------------------
Public Sub New(ByVal filename As String, ByVal irdaServiceName As String)
  '  IrDA,    
  m_IrDAServiceName = irdaServiceName

  ' ,       
  m_fileNameForDownload = filename
End Sub

'-----------------------------------------------------------
'     IR
'
' (in) filename:  ,    
'-----------------------------------------------------------
Public Sub WaitForIRFileDownloadAsync()
  ',         
  setStatus(ReceiveStatus.NotDone_SettingUp)
  '-----------------------------------------------------------
  '  
  '-----------------------------------------------------------
  Dim threadEntryPoint As System.Threading.ThreadStart
  threadEntryPoint = _
    New System.Threading.ThreadStart(AddressOf WaitForIRFileDownload)

  Dim newThread As System.Threading.Thread = _
    New System.Threading.Thread(threadEntryPoint)

  '   
  newThread.Start()
End Sub

'-----------------------------------------------------------
'  IR    
'-----------------------------------------------------------
Public Sub WaitForIRFileDownload()
  Dim outputStream As System.IO.Stream
  Dim irdaClient As System.Net.Sockets.IrDAClient
  Dim irStreamIn As System.IO.Stream

  Try
    '=========================================================
    '   !
    '=========================================================
    internal_WaitForIRFileDownload(outputStream, irdaClient, _
                irStreamIn)
  Catch '    
      setStatus(ReceiveStatus.Done_ErrorOccured)
  End Try

  '=============================================
  '  
  '=============================================
  '   
  If Not (irStreamIn Is Nothing) Then
    Try
      irStreamIn.Close()
    Catch '   
    End Try
  End If
  '  IrDA
  If Not (irdaClient Is Nothing) Then
    Try
      irdaClient.Close()
    Catch '   
    End Try
  End If
  ' ,    
  If Not (outputStream Is Nothing) Then
    Try
      outputStream.Close()
    Catch '   
    End Try
  End If
  '  ,   
  If Not (m_irListener Is Nothing) Then
    ' ,  ,   ,
    ' ,   
    m_wasListenerStopped = True
    Try
      m_irListener.Stop()
    Catch '   
    End Try
    m_irListener = Nothing
  End If
End Sub

Private Sub internal_WaitForIRFileDownload( _
    ByRef outputStream As System.IO.Stream, _
    ByRef irdaClient As System.Net.Sockets.IrDAClient, _
    ByRef irStreamIn As System.IO.Stream)
  '----------------------------------------------------------
  '        
  '----------------------------------------------------------
  outputStream = System.IO.File.Open( _
    m_fileNameForDownload, _
    System.IO.FileMode.Create)

  '==========================================
  ' 
  '==========================================
  setStatus(ReceiveStatus.NotDone_WaitingForSender)

  '-------------------------------------------------------------
  '  
  '-------------------------------------------------------------
  Try
      m_wasListenerStopped = False
    m_irListener = _
      New System.Net.Sockets.IrDAListener(m_IrDAServiceName)
    m_irListener.Start()
  Catch eListener As System.Exception
      m_errorDuringTransfer = "Error creating listener - " + _
      eListener.Message
    GoTo exit_sub_with_error
  End Try

  ',      
  If (m_wasListenerStopped = True) Then
   GoTo exit_sub_with_abort
  End If

  '---------------------------------------------------------
  ' 
  '---------------------------------------------------------
  Try
    '--------------------------------------------------------------------
    '      ,    
    '  ,     
    ',    )
    '--------------------------------------------------------------------
    irdaClient = m_irListener.AcceptIrDAClient()
  Catch eClientAccept As System.Exception
    '    ,   
    ',      
    ' . 
    If (m_wasListenerStopped = True) Then
      GoTo exit_sub_with_abort
    End If

    '    , 
    '   .  .
    m_errorDuringTransfer = "Error accepting connection - " + _
      eClientAccept.Message

    GoTo exit_sub_with_error
  End Try

  '     :
  '#1:       IR
  '#2: IR-   (-   STOP)
  '   (       )
  ',     
  If (m_wasListenerStopped = True) Then
    GoTo exit_sub_with_abort
  End If

  '==========================================
  ' 
  '==========================================
  setStatus(ReceiveStatus.NotDone_Receiving)

  '--------------------------------------------------------------
  '  
  '--------------------------------------------------------------
  Try
      irStreamIn = irdaClient.GetStream()
  Catch exGetInputStream As System.Exception
      m_errorDuringTransfer = "Error getting input stream - " + _
      exGetInputStream.Message
    GoTo exit_sub_with_error
  End Try
  '   !
  Const BUFFER_SIZE As Integer = 1024
  Dim inBuffer() As Byte
  ReDim inBuffer(BUFFER_SIZE)
  Dim bytesRead As Integer
  Do
    '    IR
    bytesRead = irStreamIn.Read(inBuffer, 0, BUFFER_SIZE)

    '     
    If (bytesRead > 0) Then
      outputStream.Write(inBuffer, 0, bytesRead)
    End If
  Loop While (bytesRead > 0)

  outputStream.Flush() '    

  '==========================================
  ' :  
  '==========================================
  setStatus(ReceiveStatus.Done_Success)
  Return 'No errors

  '==========================================
  '.
  '==========================================
exit_sub_with_abort:
  ' :  (  - )
  setStatus(ReceiveStatus.Done_Aborted)
  Return

exit_sub_with_error:

  ' : !!!!
  setStatus(ReceiveStatus.Done_ErrorOccured)
End Sub
End Class
